#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <sys/time.h>

#include "SAM.h"

using namespace std;

#define DEFAULT_SIZE 100


unsigned long long timestamp()
{
    struct timeval tp;
    gettimeofday(&tp, NULL);
    unsigned long long mslong = (unsigned long long) tp.tv_sec * 1000L + tp.tv_usec / 1000; //get current timestamp in milliseconds
    return  mslong;
}


int main (int argc, char *argv[])
{
    unsigned long prec;
    unsigned int tagDisable = SAM_ALL & ~SAM_MUL & ~SAM_DIV & ~SAM_POWER ; // & ~SAM_CANCEL
    int N = DEFAULT_SIZE;
    bool usage = false;

    /* precision is required */
    if ( argc < 2 ) { usage = true; }

    /* precision is required to be last */
    prec = strtol(argv[--argc], NULL, 10);
    /* exclude silly values */
    if ( (prec < 8) || (prec > 65535) ) { usage = true; }

    for( int posn=1; posn<argc; ++posn )
    {
        /* next argument is an option flag */
        if ( argv[posn][0] != '-' ) { usage = true; }
        char flag = argv[posn][1];
        switch ( flag )
        {
            case 'a': case 'A':
                tagDisable = 0;
                break;
            case 'n': case 'N':
                tagDisable = SAM_ALL;
                break;
            case 'v': case 'V':
                /* default tags */
                break;
            case 's': case 'S':
                /* size value is required */
                if (++posn >= argc ) { usage = true; }
                N = strtol(argv[posn], NULL, 10);
                /* exclude silly values */
                if ( (N < 0) || (N > 1000) ) { usage = true; }
                break;
            default: usage = true;
        }
    }

    if (usage)
    {
        cerr << "Usage:" << endl;
        cerr << argv[0] << ": [-s(size) <matrix_size>] [ -n(o detection) | -v(alidation) | -a(ll) ]  <binary_precision>" << endl;
        exit(-1);
    }

	cout << "----------------------------------------------------------------" << endl;
	cout << "#  Performance test for matrix multiplications with SAM" << endl; 
	cout << "#  Matrix: " << N << " * " << N << endl;
    cout << "#  Precision: " << prec << " bits." << endl;
	cout << "----------------------------------------------------------------" << endl;

	SAM_init(-1, tagDisable, prec);
	int i,j,k;
	mp_st ** amat;
	mp_st ** res;
	//	int cpt=0;
	
	unsigned long long startTime = timestamp();

	amat = new mp_st*[N];
	for(i=0; i<N; i++) {
		amat[i]=new mp_st [N];
	}
	res = new mp_st*[N];
	for(i=0; i<N; i++) {
		res[i]=new mp_st [N];
	}

	for(i=0; i<N; i++){
		for(j=0; j<N; j++){
			amat[i][j] = i+1+j;
		}
	}
	
	for(i=0; i<N; i++){
		for(j=0; j<N; j++){
			res[i][j] = 0;
			for (k=0; k<N; k++){
				res[i][j] = amat[i][k] * amat[k][j] + res[i][j];
				//	cpt++;
			}
		}
	}

	for(i=0; i<N; i++){
		delete[] amat[i];
		delete[] res[i];
	}
	delete[] amat;
	delete[] res;
	
	unsigned long long finishTime = timestamp();
	unsigned long durn = (unsigned long)(finishTime - startTime);

	SAM_end();

	cout << "----------------------------------------------------------------" << endl;
	cout << "# Execution took " << durn << " milliseconds." << endl;
	cout << "----------------------------------------------------------------" << endl;
	
	return 0;
}
