/*-----------------------------------------------------------------------
Copyright 2023 F. JEZEQUEL

This file is part of SAM.

    SAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    SAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
    Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with SAM. If not, see
    <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------*/
///////////////////////////////////////////////////
//
// SAM SOFTWARE
//
///////////////////////////////////////////////////
// Last Modifications
//
//
///////////////////////////////////////////////////
// fma operator
//
//  


#include "SAM.h"
#include "SAM_private.h"
#include "SAM_MPFR_templates.h"

using namespace SAM;


///////////////////////////////////////////////////////////////////////////////////////////

mp_st fma(const mp_st& a, const mp_st& b, const mp_st& c)
{
	mp_st res;
       	if(_SAM_mul_tag){
		if (a.accuracy==DIGIT_NOT_COMPUTED) a.nb_significant_digit();
		if (b.accuracy==DIGIT_NOT_COMPUTED) b.nb_significant_digit();
		if (a.accuracy==0 && b.accuracy==0) instability(&_SAM_mul_count);
	}
	res.accuracy=DIGIT_NOT_COMPUTED; 

	const mpfr_rnd_t xdir = (RANDOM) ? MPFR_RNDU : MPFR_RNDD;
	const bool rb2 = RANDOM;
	const mpfr_rnd_t ydir = rb2 ? MPFR_RNDU : MPFR_RNDD;
	const mpfr_rnd_t zdir = rb2 ? MPFR_RNDD : MPFR_RNDU;
	mpfr_fma( res.x, a.x, b.x, c.x, xdir );
	mpfr_fma( res.y, a.y, b.y, c.y, ydir );
	mpfr_fma( res.z, a.z, b.z, c.z, zdir );

	
  if(_SAM_cancel_tag){
    if (a.accuracy==DIGIT_NOT_COMPUTED)  a.nb_significant_digit();	      
    if (b.accuracy==DIGIT_NOT_COMPUTED)  b.nb_significant_digit();
    if (c.accuracy==DIGIT_NOT_COMPUTED)  c.nb_significant_digit();						
    if ((a.accuracy < b.accuracy ? (a.accuracy < c.accuracy ? a.accuracy : c.accuracy):
	 (b.accuracy < c.accuracy ? b.accuracy : c.accuracy)) >=		
	res.nb_significant_digit()+_SAM_cancel_value)			
      _SAM_cancel_count++;
  }

	return res;
}





// operator *=()
/*
mp_st& mp_st::operator*=(const mp_st& a)
{
	if(_SAM_mul_tag) {
		if (accuracy==DIGIT_NOT_COMPUTED) nb_significant_digit();
		if (accuracy==0){
			if (a.accuracy==DIGIT_NOT_COMPUTED) a.nb_significant_digit();
			if (a.accuracy==0) {
				instability(&_SAM_mul_count);
			}
		}
	}

	const mpfr_rnd_t xdir = (RANDOM) ? MPFR_RNDU : MPFR_RNDD;
	const bool rb2 = RANDOM;
	const mpfr_rnd_t ydir = rb2 ? MPFR_RNDU : MPFR_RNDD;
	const mpfr_rnd_t zdir = rb2 ? MPFR_RNDD : MPFR_RNDU;
	SAM::mul_mpfr( x, a.x, xdir );
	SAM::mul_mpfr( y, a.y, ydir );
	SAM::mul_mpfr( z, a.z, zdir );

	if (_SAM_cancel_tag) {
		this->accuracy = this->nb_significant_digit(); // preemptive calculate
	} else {
		this->accuracy = DIGIT_NOT_COMPUTED;
	}
	return *this;
	
}
*/

